@@ -16,12 +16,13 @@ This module provides bases classes for tables management.
 """
 
 import json
-
 from pyramid.decorator import reify
 from zope.component import queryMultiAdapter
 from zope.interface import implementer
 from zope.schema.fieldproperty import FieldProperty
 
+from pyams_security.interfaces import ISecurityContext
+from pyams_security.permission import get_edit_permission, get_permission_checker
 from pyams_security.security import ProtectedViewObjectMixin
 from pyams_table.column import Column, GetAttrColumn
 from pyams_table.table import Table as BaseTable, get_weight
@@ -29,13 +30,14 @@ from pyams_template.template import get_view_template
 from pyams_utils.adapter import ContextRequestViewAdapter
 from pyams_utils.data import ObjectDataManagerMixin
 from pyams_utils.date import SH_DATE_FORMAT, format_datetime
-from pyams_utils.factory import get_object_factory, is_interface
+from pyams_utils.factory import get_object_factory
 from pyams_utils.interfaces import ICacheKeyValue
 from pyams_utils.interfaces.data import IObjectData
 from pyams_utils.list import boolean_iter
 from pyams_utils.url import absolute_url
+from pyams_viewlet.viewlet import ViewContentProvider
 from pyams_zmi.interfaces.table import IInnerTable, IMultipleTableView, IReorderColumn, \
-    ITableAdminView, ITableElementEditor, ITableView
+    ITableAdminView, ITableElementEditor, ITableGroupSwitcher, ITableView
 from pyams_zmi.utils import get_object_hint, get_object_icon, get_object_label
 from pyams_zmi.view import InnerAdminView
 
@@ -112,18 +114,25 @@ def get_ordered_data_attributes(source, container, request, target='reorder.json
         'data-searching': 'false',
         'data-info': 'false',
         'data-paging': 'false',
-        'data-ams-order': '0,asc',
-        'data-row-reorder': '{"update": false}',
-        'data-ams-location': absolute_url(container, request),
-        'data-ams-reorder-url': target
-    })
-    source.setdefault('tr', {}).update({
-        'data-ams-row-value': lambda row, col: get_row_name(row)
-    })
-    source.setdefault('td', {}).update({
-        'data-order': lambda x, col: list(container.keys()).index(x.__name__)
-            if IReorderColumn.providedBy(col) else None
+        'data-ams-order': '0,asc'
     })
+    context = ISecurityContext(container, None)
+    if context is None:
+        context = request.context
+    permission = get_edit_permission(request, context)
+    if (not permission) or request.has_permission(permission, context=container):
+        source.setdefault('table', {}).update({
+            'data-ams-location': absolute_url(container, request),
+            'data-row-reorder': '{"update": false}',
+            'data-ams-reorder-url': target
+        })
+        source.setdefault('tr', {}).update({
+            'data-ams-row-value': lambda row, col: get_row_name(row)
+        })
+        source.setdefault('td', {}).update({
+            'data-order': lambda x, col: list(container.keys()).index(x.__name__)
+                if IReorderColumn.providedBy(col) else None
+        })
 
 
 class Table(ObjectDataManagerMixin, BaseTable):
@@ -212,13 +221,28 @@ class InnerTableMixin:
     table_class = Table
     table_label = FieldProperty(ITableAdminView['table_label'])
 
+    container_intf = None
+    container_name = ''
+
     empty_template = get_view_template(name='empty')
 
     def __init__(self, context, request, *args, **kwargs):
         super().__init__(context, request, *args, **kwargs)
-        factory = get_object_factory(self.table_class) if is_interface(self.table_class) \
-            else self.table_class
-        self.table = factory(context, request)
+        container = self.get_container()
+        self.table = self.get_table(container)
+
+    def get_container(self):
+        """Table container getter"""
+        if self.container_intf is None:
+            return self.context
+        registry = self.request.registry
+        return registry.queryAdapter(self.context, self.container_intf,
+                                     name=self.container_name)
+
+    def get_table(self, container):
+        """Table factory"""
+        factory = get_object_factory(self.table_class)
+        return factory(container, self.request)
 
     def update(self):
         """Admin view updater"""
@@ -281,6 +305,39 @@ class MultipleTablesAdminView(MultipleTablesMixin, InnerAdminView):
     """Multiple tables admin view"""
 
 
+@implementer(ITableGroupSwitcher)
+class TableGroupSwitcher(ObjectDataManagerMixin, InnerTableAdminView, ViewContentProvider):
+    """Table group switcher
+
+    This switcher group is very similar to the form group switcher, but is only
+    used to handle an inner table.
+    """
+
+    legend = FieldProperty(ITableGroupSwitcher['legend'])
+    minus_class = FieldProperty(ITableGroupSwitcher['minus_class'])
+    plus_class = FieldProperty(ITableGroupSwitcher['plus_class'])
+    switcher_mode = FieldProperty(ITableGroupSwitcher['switcher_mode'])
+
+    @property
+    def state(self):
+        """Current state getter"""
+        if self.switcher_mode == 'always':
+            return 'open'
+        if self.switcher_mode == 'never':
+            return 'closed'
+        # else: automatic mode
+        has_values, _values = boolean_iter(self.table.values)
+        return 'open' if has_values else 'closed'
+
+    def get_forms(self, include_self=True):
+        """Forms getter
+
+        This method doesn't return anything, but  is used when component is used as
+        an inner form adapter!
+        """
+        yield from ()
+
+
 class I18nColumnMixin:
     """Column mixin with I18n header"""
 
@@ -293,18 +350,19 @@ class I18nColumnMixin:
 
 
 @implementer(IReorderColumn)
-class ReorderColumn(Column):
+class ReorderColumn(ProtectedViewObjectMixin, Column):
     """Reorder column"""
 
     weight = 0
     sortable = 'false'
 
-    css_classes = {
-        'th': 'reorder action',
-        'td': 'sorter action mouse-move'
-    }
-
-    permission = None
+    @property
+    def css_classes(self):
+        has_permission = self.has_permission(self.context)
+        return {
+            'th': f"{'reorder ' if has_permission else ''}action",
+            'td': f"{'sorter mouse-move ' if has_permission else ''}action"
+        }
 
     def has_permission(self, item):
         """Column permission test"""
@@ -485,27 +543,59 @@ class JsActionColumn(ActionColumn):
         return self.href
 
 
-class VisibilityColumn(ObjectDataManagerMixin, JsActionColumn):
-    """Visibility switcher column"""
-
-    hint = _("Click icon to show or hide item")
+class AttributeSwitcherColumn(ObjectDataManagerMixin, JsActionColumn):
+    """Attribute switcher column"""
 
     href = 'MyAMS.container.switchElementAttribute'
     modal_target = False
 
-    object_data = {
-        'ams-modules': 'container',
-        'ams-update-target': 'switch-visible-item.json',
-        'ams-attribute-name': 'visible',
-        'ams-icon-on': 'far fa-eye',
-        'ams-icon-off': 'far fa-eye-slash'
-    }
+    attribute_name = None
+    attribute_switcher = None
 
-    weight = 1
+    icon_on_class = 'far fa-eye'
+    icon_off_class = 'far fa-eye-slash'
+
+    @property
+    def object_data(self):
+        """Object data getter"""
+        return {
+            'ams-modules': 'container',
+            'ams-update-target': self.attribute_switcher,
+            'ams-attribute-name': self.attribute_name,
+            'ams-icon-on': self.icon_on_class,
+            'ams-icon-off': self.icon_off_class
+        }
 
     def get_icon_class(self, item):
         """Icon class getter"""
-        return 'far fa-eye' if item.visible else 'far fa-eye-slash'
+        return self.icon_on_class if getattr(item, self.attribute_name) else self.icon_off_class
+
+    def render_cell(self, item):
+        """Cell renderer"""
+        if self.has_permission(item):
+            return super().render_cell(item)
+        return f'<span class="hint" data-original-title="{self.get_icon_hint(item)}">' \
+               f'{self.get_icon(item)}' \
+               f'</span>'
+
+
+class VisibilityColumn(AttributeSwitcherColumn):
+    """Visibility switcher column"""
+
+    attribute_name = 'visible'
+    attribute_switcher = 'switch-visible-item.json'
+
+    weight = 1
+
+    def get_icon_hint(self, item):
+        """Icon hint getter"""
+        if self.has_permission(item):
+            hint = _("Click to show/hide item")
+        elif item.visible:
+            hint = _("This element is visible")
+        else:
+            hint = _("This element is not visible")
+        return self.request.localizer.translate(hint)
 
 
 class TrashColumn(ObjectDataManagerMixin, JsActionColumn):
@@ -525,6 +615,15 @@ class TrashColumn(ObjectDataManagerMixin, JsActionColumn):
 
     weight = 999
 
+    def has_permission(self, item):
+        """Column permission checker"""
+        request = self.request
+        checker = get_permission_checker(request, item, action=self.action_type)
+        permission = checker.edit_permission if checker is not None else None
+        if permission is not None:
+            return request.has_permission(permission, context=item)
+        return super().has_permission(item)
+
 
 class DateColumn(GetAttrColumn):
     """Date or datetime column"""
